/*
 *  This file is
 *    Copyright (C) 2001 Alexey Klimkin <klimkin@mcst.ru>
 *
 *  It is part of httppc, which is
 *    Copyright (C) 2001 Alexey Klimkin <klimkin@mcst.ru>
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2, or (at your option)
 *  any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software Foundation,
 *  Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */

#ifndef __HTTPPC__H__
#define __HTTPPC__H__

#include <sys/types.h>
#include <sys/socket.h>
#include <netdb.h>
#include <sys/time.h>

struct addrinfo;
struct sockaddr;
struct hostent;
struct pollfd;

typedef enum {
    httppc_none         = 0x00,
    httppc_nocache      = 0x01,
    httppc_keepalive    = 0x02,
    httppc_auth         = 0x04,
    httppc_useragent    = 0x08,
    httppc_ldns         = 0x10, // driver mode only
} httppc_args;

typedef int (*httppc_connect_t)(int sockfd,
                                const struct sockaddr * serv_addr,
                                socklen_t addrlen);
typedef struct hostent *(*httppc_gethostbyname_t)(const char *name);
typedef int (*httppc_getaddrinfo_t)(const char *node, const char *service,
                                    const struct addrinfo * hints,
                                    struct addrinfo ** res);
// Linux
#if defined(__GLIBC__)
typedef int (*httppc_gethostbyname_r_t)(const char *name, struct hostent *result_buf, 
                                        char *buf, size_t buflen, 
                                        struct hostent **result, int *h_errnop);
typedef int (*httppc_gethostbyaddr_r_t)(const char *addr, socklen_t len, int type,
                                        struct hostent *result_buf, 
                                        char *buf, size_t buflen, 
                                        struct hostent **result, int *h_errnop);
typedef struct hostent * (*httppc_gethostbyaddr_t)(const void *addr, socklen_t len, int type);
// Solaris
#else
typedef struct hostent * (*httppc_gethostbyname_r_t)(const char *name, struct hostent *result_buf, 
                                                     char *buf, int buflen, int *h_errnop);
typedef struct hostent * (*httppc_gethostbyaddr_r_t)(const char *addr, int len, int type,
                                                     struct hostent *result_buf, 
                                                     char *buf, int buflen, int *h_errnop);
typedef struct hostent * (*httppc_gethostbyaddr_t)(const char *addr, int len, int type);
#endif
typedef int (*httppc_socket_t)(int domain, int type, int protocol);
typedef int (*httppc_setsockopt_t)(int s, int level, int optname, const void *optval, int optlen);
typedef int (*httppc_send_t)(int s, const void *msg, int len, unsigned int flags);
typedef int (*httppc_sendto_t)(int s, const void *msg, int len, unsigned int flags, const struct sockaddr *to, int tolen);
typedef int (*httppc_sendmsg_t)(int s, const struct msghdr *msg, unsigned int flags);

typedef int (*httppc_select_t)(int n, fd_set *readfds, fd_set *writefds, fd_set *exceptfds, struct timeval *timeout);
typedef int (*httppc_poll_t)(struct pollfd *ufds, unsigned int nfds, int timeout);

typedef struct
{
    // system entries
    httppc_connect_t real_connect;
    httppc_gethostbyname_t real_gethostbyname;
    httppc_gethostbyname_r_t real_gethostbyname_r;
    // proxy parameters
    const char *http_proxy;
    unsigned short http_proxy_port;
    // HTTP proxy authorization parameters
    const char *proxy_user, *proxy_pass;
    // user agent
    const char *useragent;
    // connect timeout
    int timeout;
    // initialized arguments
    httppc_args args;
} httppc_options_t;

#ifdef __cplusplus
extern "C" {
#endif

/* create TCP/IP connection through HTTP proxy http_proxy:http_proxy_port
   
   sockfd should be in SOCK_STREAM mode.
   
   http_head_lines specifies format for additional lines to add to HTTP
   request for connections.
   */
int httppc_connect(int sockfd, const struct sockaddr *serv_addr, int addrlen,
                   const httppc_options_t *options);
int httppc_connect2(int sockfd, const char *serv_name, unsigned short serv_port, int in_family,
                    const httppc_options_t *options);

/* get DNS information from host extern_dns via adns library
   this function doesn't actually traverse via HTTP proxy */
struct hostent * httppc_gethostbyname(const char *name, const char *extern_dns);
struct hostent * httppc_gethostbyname_r(const char *name, struct hostent *result_buf, 
										char *buf, int buflen, int *h_errnop,
										const char *extern_dns);
struct hostent * httppc_gethostbyaddr(const void *addr, int len, int type, 
                                      const char *extern_dns);
struct hostent * httppc_gethostbyaddr_r(const void *addr, int len, int type,
                                        struct hostent *result_buf, 
                                        char *buf, int buflen, int *h_errnop,
                                        const char *extern_dns);

#ifdef __cplusplus
};
#endif

#endif

